package org.artifactory.queue;

import org.artifactory.event.EventType;
import org.artifactory.event.queue.QueueEvent;
import org.artifactory.repo.RepoPath;
import org.artifactory.repo.RepoPathFactory;
import org.artifactory.storage.event.EventInfo;

import java.util.Objects;
import java.util.Set;

public class QueueEventImpl implements QueueEvent {

    private final EventInfo event;
    private QueueEvent originalEvent;
    private final RepoPath repoPath;

    public QueueEventImpl(EventInfo event) {
        this.event = event;
        repoPath = RepoPathFactory.create(getPath());
    }

    public QueueEventImpl(QueueEventImpl event, QueueEventImpl originalEvent) {
        this(event.getEvent());
        this.originalEvent = originalEvent;
    }

    @Override
    public long getEventId() {
        return event.getEventId();
    }

    @Override
    public Set<String> getHints() {
        return event.getHints();
    }

    @Override
    public String getRepoKey() {
        return repoPath.getRepoKey();
    }

    @Override
    public String getArtifactPath() {
        return repoPath.getPath();
    }

    @Override
    public String getPath() {
        return event.getPath();
    }

    @Override
    public EventType getType() {
        return event.getType();
    }

    @Override
    public long getTimestamp() {
        return event.getTimestamp();
    }

    public EventInfo getEvent() {
        return event;
    }

    public QueueEvent getOriginalEvent() {
        return originalEvent;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {return true;}
        if (o == null || getClass() != o.getClass()) {return false;}
        QueueEventImpl event = (QueueEventImpl) o;
        return getEventId() == event.getEventId() && getType() == event.getType() &&
                Objects.equals(getPath(), event.getPath()) && Objects.equals(getHints(), event.getHints());
    }

    @Override
    public int hashCode() {
        return Objects.hash(getEventId(), getPath(), getPath(), getHints());
    }

    @Override
    public String toString() {
        return getTimestamp() + "|" + getType() + "|" + getPath();
    }
}
