package org.artifactory.release.bundle;

import org.artifactory.api.rest.distribution.bundle.models.ArtifactsBundleModel;
import org.artifactory.storage.db.bundle.model.BundleNode;
import org.jfrog.common.StreamSupportUtils;

import java.sql.SQLException;
import java.util.Comparator;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * A release bundle delete template to avoid read-lock related deadlocks with Derby DB
 *
 * @author Omri Ziv
 */
public class DerbyReleaseBundleDeleteTemplate implements ReleaseBundleDeleteTemplate {
    private InternalReleaseBundleService internalReleaseBundleService;

    public DerbyReleaseBundleDeleteTemplate(InternalReleaseBundleService internalReleaseBundleService) {
        this.internalReleaseBundleService = internalReleaseBundleService;
    }

    @Override
    public void deleteBundle(ArtifactsBundleModel artifactsBundle, List<BundleNode> bundleNodes,
            Set<Long> nodeIdsWithArtifactsToDelete) throws SQLException {
        internalReleaseBundleService.deleteBundleNodesInnerTransaction(artifactsBundle);
        List<BundleNode> sortedBundleNodes = StreamSupportUtils.stream(bundleNodes)
                .sorted(Comparator.comparing(BundleNode::getRepoPath))
                .collect(Collectors.toList());

        for (BundleNode bundleNode : sortedBundleNodes) {
            internalReleaseBundleService.undeployIfNeeded(artifactsBundle, nodeIdsWithArtifactsToDelete, bundleNode);
        }
    }

}
