package org.artifactory.queue.behaviors;

import com.google.common.collect.Lists;
import org.artifactory.queue.QueueEventImpl;

import java.util.List;

import static org.artifactory.event.EventType.create;
import static org.artifactory.event.EventType.props;
import static org.artifactory.event.EventType.update;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertThrows;

/**
 * @author avishayh
 */
public class PrioritizedEventDedupBehaviourTest extends AbstractEventReductionBehaviorTest {

    public void prioritizeWith2Types() {
        PrioritizedEventDedupBehavior behavior = new PrioritizedEventDedupBehavior(create, update);
        List<QueueEventImpl> before;
        List<QueueEventImpl> after;

        before = Lists.newArrayList();
        add(before, create, "a"); //removed
        add(before, update, "a"); //merged with create
        after = behavior.apply(before);
        assertEquals(after.size(), 1);
        assertEquals(after.get(0).toString(), "2|create|a", "event merged: last id with prioritized type");
        assertEquals(((QueueEventImpl) after.get(0)).getOriginalEvent().getEventId(), 1,
                "the original eventId");
        reset();
        before = Lists.newArrayList();
        add(before, update, "a"); //removed
        add(before, create, "a"); //regular dedup - create is higher priority
        after = behavior.apply(before);
        assertEquals(after.size(), 1);
        assertEquals(after.get(0).toString(), "2|create|a", "no merge needed");
        reset();
        before = Lists.newArrayList();
        add(before, create, "a"); //removed
        add(before, create, "b");
        add(before, update, "a"); //merged with create
        after = behavior.apply(before);
        assertEquals(after.size(), 2);
        assertEquals(after.get(0).toString(), "2|create|b");
        assertEquals(after.get(1).toString(), "3|create|a", "event merged: last id with prioritized type");
        assertEquals(after.get(1).getOriginalEvent().getEventId(), 1, "the original eventId");

        reset();
        before = Lists.newArrayList();
        add(before, create, "a"); //removed
        add(before, update, "a"); //removed
        add(before, update, "a"); //merged with create
        after = behavior.apply(before);
        assertEquals(after.size(), 1);
        assertEquals(after.get(0).toString(), "3|create|a", "event merged: last id with prioritized type");
        assertEquals(after.get(0).getOriginalEvent().getEventId(), 1, "the original eventId");

        reset();
        before = Lists.newArrayList();
        add(before, create, "a");
        add(before, props, "a"); ///props interfere in the middle of create and update
        add(before, update, "a");
        after = behavior.apply(before);
        assertEquals(after.size(), 3);
        assertEquals(after, before, "props interfere in the middle of create and update");
        reset();
        before = Lists.newArrayList();
        add(before, create, "a");
        add(before, props, "a");
        add(before, props, "a");
        add(before, update, "a");
        add(before, update, "a");
        add(before, props, "a");
        add(before, update, "a"); //last event is update, so props still in the middle
        after = behavior.apply(before);
        assertEquals(after.size(), 3);
        assertEquals(after.get(0).toString(), "1|create|a");
        assertEquals(after.get(1).toString(), "6|props|a");
        assertEquals(after.get(2).toString(), "7|update|a");
        reset();
        before = Lists.newArrayList();
        add(before, create, "a");
        add(before, props, "a");
        add(before, props, "a");
        add(before, create, "b");
        add(before, update, "a");
        add(before, update, "a"); //create will be merged to here
        add(before, props, "a");  //props will be after crete
        after = behavior.apply(before);
        assertEquals(after.size(), 3);
        assertEquals(after.get(0).toString(), "4|create|b");
        assertEquals(after.get(1).toString(), "6|create|a");
        assertEquals(after.get(2).toString(), "7|props|a");
    }

    public void prioritizeWith3Types() {
        EventReductionBehavior behavior = new FederatedDedupBehavior();
        List<QueueEventImpl> before;
        List<QueueEventImpl> after;

        before = Lists.newArrayList();
        add(before, update, "b");
        add(before, create, "a");
        add(before, props, "a");
        add(before, props, "a");
        add(before, props, "b");
        add(before, update, "a");
        add(before, update, "a");
        add(before, props, "a");
        after = behavior.apply(before);
        assertEquals(after.size(), 2);
        assertEquals(after.get(0).toString(), "5|update|b", "regular prioritize dedup");
        assertEquals(after.get(1).toString(), "8|create|a", "merged prioritize dedup");
        assertEquals(after.get(1).getOriginalEvent().getEventId(), 2, "the original eventId");
    }

    public void illegalDedupBehaviour() {
        //must crete with at least 2 types (0 or 1 are not illegal)
        assertThrows(IllegalArgumentException.class, PrioritizedEventDedupBehavior::new);
        assertThrows(IllegalArgumentException.class, () -> new PrioritizedEventDedupBehavior(update));
    }
}
