package org.artifactory.repo.service.mover;

import org.apache.http.HttpStatus;
import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.replication.ReplicationAddon;
import org.artifactory.api.common.MoveMultiStatusHolder;
import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.repo.LocalRepo;
import org.artifactory.repo.RepoPath;
import org.artifactory.repo.RepoRepoPath;
import org.artifactory.repo.StoringRepo;
import org.artifactory.sapi.fs.VfsItem;
import org.artifactory.storage.db.fs.model.DbMutableFile;
import org.artifactory.storage.db.fs.session.SqlStorageSession;
import org.artifactory.storage.fs.MutableVfsItem;
import org.artifactory.storage.fs.lock.SessionLockEntry;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.testng.Assert;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import static org.artifactory.repo.service.flexible.tools.MoveCopyTestUtils.prepareCopyOrMoveOverrideMocks;
import static org.artifactory.repo.service.flexible.tools.MoveCopyTestUtils.prepareDbMutableFileMock;
import static org.mockito.Mockito.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.verifyNoMoreInteractions;
import static org.mockito.Mockito.when;
import static org.mockito.Mockito.withSettings;

public class BaseRepoPathMoverTest extends ArtifactoryHomeBoundTest {

    @Mock
    RepoPath repoPathMock;
    @Mock
    LocalRepo repoMock;
    @Mock
    MutableVfsItem mutableVfsItemMock;
    @Mock
    SqlStorageSession sqlStorageSessionMock;
    @Mock
    SessionLockEntry sessionLockEntryMock;
    @Mock
    MoveMultiStatusHolder moveMultiStatusHolderMock;
    @Mock
    private ArtifactoryContext context;

    @BeforeMethod
    public void setup() {
        MockitoAnnotations.openMocks(this);
        ArtifactoryContextThreadBinder.bind(context);
    }

    @AfterMethod
    public void tearDown() {
        ArtifactoryContextThreadBinder.unbind();
    }

    @Test
    public void testMoveFile() {
        BaseRepoPathMover baseRepoPathMove = mock(BaseRepoPathMover.class, Mockito.CALLS_REAL_METHODS);
        DbMutableFile dbMutableFile = prepareDbMutableFileMock();
        RepoRepoPath<StoringRepo> repoRepoPath =
                prepareCopyOrMoveOverrideMocks(repoPathMock, repoMock, mutableVfsItemMock, sessionLockEntryMock,
                        sqlStorageSessionMock, moveMultiStatusHolderMock, true);

        baseRepoPathMove.moveFile(dbMutableFile, repoRepoPath, moveMultiStatusHolderMock, true);

        verify(repoMock, times(1)).createOrGetFile(any());
        verify(sqlStorageSessionMock, times(1)).save();
        verify(moveMultiStatusHolderMock, times(1)).artifactMoved();
    }

    @Test
    public void testCanMoveWithoutReadPermissionToSourceRepository() {
        RepoPath sourceRepoPath = mock(RepoPath.class);
        VfsItem item = mock(VfsItem.class);
        when(item.getRepoPath()).thenReturn(sourceRepoPath);
        AuthorizationService authorizationService = mock(AuthorizationService.class);
        when(authorizationService.canRead(sourceRepoPath)).thenReturn(false);
        when(context.getAuthorizationService()).thenReturn(authorizationService);
        AddonsManager addonsManager = mock(AddonsManager.class);
        when(addonsManager.addonByType(ReplicationAddon.class)).thenReturn(mock(ReplicationAddon.class));
        when(context.beanForType(AddonsManager.class)).thenReturn(addonsManager);
        BaseRepoPathMover baseRepoPathMove = mock(BaseRepoPathMover.class,
                withSettings().useConstructor(new MoveMultiStatusHolder(), mock(MoverConfig.class)));
        when(baseRepoPathMove.canMove(any(), any())).thenCallRealMethod();
        Assert.assertFalse(baseRepoPathMove.canMove(item, null));
        Assert.assertEquals(baseRepoPathMove.status.getStatusCode(), HttpStatus.SC_FORBIDDEN);
        Assert.assertTrue(baseRepoPathMove.status.getStatusMsg().contains("User doesn't have permissions to read"));
        verify(item, times(1)).getRepoPath();
        verify(authorizationService, times(1)).canRead(sourceRepoPath);
        verify(context, times(1)).getAuthorizationService();
        verifyNoMoreInteractions(item, authorizationService);
    }

}