package org.artifactory.addon.observability;

import org.testng.annotations.Test;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.Map;

import static org.artifactory.addon.observability.ObservabilityInfo.LOG_NAME_FIELD;
import static org.artifactory.addon.observability.ObservabilityInfo.PRODUCT_NAME_FIELD;
import static org.artifactory.addon.observability.ObservabilityInfo.TYPE_FIELD;
import static org.artifactory.addon.observability.UsageLogShippingInfo.NODE_ID_FIELD;
import static org.artifactory.addon.observability.UsageLogShippingInfo.SERVICE_ID_FIELD;
import static org.testng.Assert.assertEquals;

/**
 * @author Andrey Tashlitsky
 */
public class UsageLogShippingInfoTest {

    private static final String COOL_PRODUCT = "cool-product";
    private static final String COOL_TYPE = "cool-type";
    private static final String COOL_LOG_NAME = "cool-log-name";
    private static final String COOL_SERVICE_ID = "cool-service-id";
    private static final String COOL_NODE_ID = "cool-node-id";
    private static final InputStream COOL_INPUT_STREAM = new ByteArrayInputStream("cool-input-stream".getBytes());

    @Test
    public void testBuilder() {
        UsageLogShippingInfo usageLogShippingInfo = UsageLogShippingInfo.builder()
                .productName(COOL_PRODUCT)
                .type(COOL_TYPE)
                .logName(COOL_LOG_NAME)
                .serviceId(COOL_SERVICE_ID)
                .nodeId(COOL_NODE_ID)
                .inputStream(COOL_INPUT_STREAM)
                .build();
        assertEquals(usageLogShippingInfo.getProductName(), COOL_PRODUCT);
        assertEquals(usageLogShippingInfo.getType(), COOL_TYPE);
        assertEquals(usageLogShippingInfo.getLogName(), COOL_LOG_NAME);
        assertEquals(usageLogShippingInfo.getServiceId(), COOL_SERVICE_ID);
        assertEquals(usageLogShippingInfo.getNodeId(), COOL_NODE_ID);
        assertEquals(usageLogShippingInfo.getInputStream(), COOL_INPUT_STREAM);
    }

    @Test
    public void testGetProperties() {
        UsageLogShippingInfo usageLogShippingInfo = UsageLogShippingInfo.builder()
                .productName(COOL_PRODUCT)
                .type(COOL_TYPE)
                .logName(COOL_LOG_NAME)
                .serviceId(COOL_SERVICE_ID)
                .nodeId(COOL_NODE_ID)
                .inputStream(COOL_INPUT_STREAM)
                .build();
        Map<String, String> properties = usageLogShippingInfo.getProperties();
        assertEquals(properties.size(), 5);
        assertEquals(properties.get(LOG_NAME_FIELD), COOL_LOG_NAME);
        assertEquals(properties.get(PRODUCT_NAME_FIELD), COOL_PRODUCT);
        assertEquals(properties.get(TYPE_FIELD), COOL_TYPE);
        assertEquals(properties.get(NODE_ID_FIELD), COOL_NODE_ID);
        assertEquals(properties.get(SERVICE_ID_FIELD), COOL_SERVICE_ID);
    }
}
