package org.artifactory.ui.rest.model.admin.services.indexer;

import com.google.common.collect.Maps;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.descriptor.config.CentralConfigDescriptor;
import org.artifactory.descriptor.index.IndexerDescriptor;
import org.artifactory.descriptor.repo.FederatedRepoDescriptor;
import org.artifactory.descriptor.repo.RepoBaseDescriptor;
import org.artifactory.descriptor.repo.RepoType;
import org.jetbrains.annotations.NotNull;
import org.testng.Assert;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.Test;

import java.util.List;
import java.util.Map;
import java.util.SortedSet;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

/**
 * @author Alexei Vainshtein
 */
public class IndexerTest {

    @AfterMethod
    public void tearDown() {
        ArtifactoryContextThreadBinder.unbind();
    }

    @Test
    public void testPopulateRepos() {
        ArtifactoryContext context = mock(ArtifactoryContext.class);
        ArtifactoryContextThreadBinder.bind(context);
        Map<String, FederatedRepoDescriptor> federatedRepositoriesMap = prepareDescriptor(context);
        Indexer indexer = new Indexer();
        indexer.populateRepos(null);
        Assert.assertEquals(indexer.getExcludedRepos().size(), federatedRepositoriesMap.size());
    }

    @Test
    public void testPopulateDescriptors() {
        ArtifactoryContext context = mock(ArtifactoryContext.class);
        ArtifactoryContextThreadBinder.bind(context);
        Map<String, FederatedRepoDescriptor> federatedRepositoriesMap = prepareDescriptor(context);
        Indexer indexer = new Indexer();
        SortedSet<RepoBaseDescriptor> repoBaseDescriptors =
                indexer.populateDescriptors(List.of("fed-maven", "fed-gradle"), Maps.newLinkedHashMap(),
                        populateFederatedReposMap(), Maps.newLinkedHashMap(), Maps.newLinkedHashMap());
        Assert.assertEquals(repoBaseDescriptors.size(), federatedRepositoriesMap.size());
    }

    @Test
    public void testToDescriptor() {
        ArtifactoryContext context = mock(ArtifactoryContext.class);
        ArtifactoryContextThreadBinder.bind(context);
        Map<String, FederatedRepoDescriptor> federatedRepositoriesMap = prepareDescriptor(context);

        Indexer indexer = new Indexer();
        indexer.setIncludedRepos(List.of("fed-maven", "fed-gradle"));
        IndexerDescriptor indexerDescriptor = indexer.toDescriptor();
        assert indexerDescriptor.getIncludedRepositories() != null;
        Assert.assertEquals(indexerDescriptor.getIncludedRepositories().size(), federatedRepositoriesMap.size());

    }

    private Map<String, FederatedRepoDescriptor> prepareDescriptor(ArtifactoryContext context) {
        CentralConfigService configService = mock(CentralConfigService.class);
        CentralConfigDescriptor configDescriptor = mock(CentralConfigDescriptor.class);
        Map<String, FederatedRepoDescriptor> federatedRepositoriesMap = populateFederatedReposMap();
        when(configDescriptor.getFederatedRepositoriesMap()).thenReturn(federatedRepositoriesMap);
        when(configService.getDescriptor()).thenReturn(configDescriptor);
        when(context.beanForType(CentralConfigService.class)).thenReturn(configService);
        return federatedRepositoriesMap;
    }

    @NotNull
    private Map<String, FederatedRepoDescriptor> populateFederatedReposMap() {
        FederatedRepoDescriptor repoDescriptor = new FederatedRepoDescriptor();
        repoDescriptor.setKey("fed-maven");
        repoDescriptor.setType(RepoType.Maven);
        Map<String, FederatedRepoDescriptor> federatedRepositoriesMap = Maps.newLinkedHashMap();
        federatedRepositoriesMap.put(repoDescriptor.getKey(), repoDescriptor);
        repoDescriptor = new FederatedRepoDescriptor();
        repoDescriptor.setKey("fed-gradle");
        repoDescriptor.setType(RepoType.Gradle);
        federatedRepositoriesMap.put(repoDescriptor.getKey(), repoDescriptor);
        return federatedRepositoriesMap;
    }
}